#!/usr/bin/bash

# Copyright (c) 2019 Oracle and/or its affiliates. All rights reserved.

TUNED_LOG=/var/log/tuned/tuned.log

# recommended setting of combined channels
NUM_CHANNELS=16

. /usr/lib/tuned/functions

log_info() {
	echo "$(date +'%F %T,%3N') INFO     tuned profile 'oci-nic': $1" >> $TUNED_LOG
}

start() {
	# check if running on a virtual machine
	vir_what=$(virt-what 2>/dev/null)

	# support this change only on BM shapes that use the bnxt_en driver
	if [[ ! -z $vir_what  ]]; then
		log_info "not on BM, so NIC settings are not changed"
		return 0
	fi

	# the NICs that are up
	net_infs=$(ip addr | awk '/state UP/ {print $2}'| sed 's/.$//')

	if [[ -z $net_infs ]]; then
		log_info "on BM, but no NIC is up"
		return 0
	fi

	# for each NIC that is up
	for inf in $net_infs; do
		inf_driver=$(ethtool -i $inf 2>/dev/null | grep -i driver | awk -F ': ' '{print $2}')
		old_chnnls=''
		# change the setting of combined channels as needed
		if [[ ! -z $inf_driver ]] && [[ $inf_driver = "bnxt_en" ]]; then
			old_chnnls=$(ethtool -l $inf 2>/dev/null | grep -i combined | tail -1 | awk -F ':\t' '{print $2}')
			if [[ ! -z $old_chnnls ]] && [[ $old_chnnls -lt $NUM_CHANNELS ]]; then
				ethtool -L $inf combined $NUM_CHANNELS
			fi
		fi
		new_chnnls=$(ethtool -l $inf 2>/dev/null | grep -i combined | tail -1 | awk -F ':\t' '{print $2}')
		if [[ ! -z $old_chnnls ]] && [[ $new_chnnls != $old_chnnls ]]; then
			log_info "NIC $inf: driver $inf_driver, combined channels changed from $old_chnnls to $new_chnnls"
		else
			log_info "NIC $inf: driver $inf_driver, combined channels $new_chnnls"
		fi
	done

	return 0
}

stop() {
	# Do nothing

	return 0
}

process $@
